# -*- coding: utf-8 -*-
#
# Copyright © 2014 Geoffrey T. Dairiki <dairiki@dairiki.org>
#
from __future__ import absolute_import

import logging

from pyramid.request import Request
from pyramid import testing
import pytest

@pytest.fixture
def current_request(request):
    r = Request.blank('/')
    config = testing.setUp(request=r)
    request.addfinalizer(testing.tearDown)
    return r

@pytest.fixture
def log_record():
    return logging.LogRecord('test', logging.INFO, __file__, 0, '', (), None)

class MockObject(object):
    def __init__(self, **kw):
        self.__dict__.update(kw)

class TestFormatter(object):
    def make_one(self, *args, **kwargs):
        from pyramid_log import Formatter
        return Formatter(*args, **kwargs)

    def test_with_explicit_request(self, log_record):
        log_record.request = Request.blank('/', POST={})
        formatter = self.make_one('%(request.method)s')
        assert formatter.format(log_record) == 'POST'

    def test_with_threadlocal_request(self, current_request, log_record):
        formatter = self.make_one('%(request.method)s')
        assert formatter.format(log_record) == 'GET'

class TestReplaceDict(object):
    def make_one(self, obj, d):
        from pyramid_log import _ReplaceDict
        return _ReplaceDict(obj, d)

    def test_getattr(self):
        class Obj(object):
            def m(self):
                return 'foo'
        obj = Obj()
        proxy = self.make_one(obj, {'x': 'bar'})
        assert proxy.m() == 'foo'
        assert proxy.x == 'bar'

    def test_setattr_modifies_proxy(self):
        obj = MockObject(x='orig')
        d = {}
        proxy = self.make_one(obj, d)
        proxy.x = 'changed'
        assert d['x'] == 'changed'
        assert obj.x == 'orig'

    def test_init_with_explicit_dict(self):
        obj = object()
        d = {}
        proxy = self.make_one(obj, d)
        assert proxy.__dict__ is d

class TestChainingDict(object):
    def make_one(self, *args, **kwargs):
        from pyramid_log import _ChainingDict
        return _ChainingDict(*args, **kwargs)

    def test_chained_getitem(self):
        d = self.make_one({'a': {'b': 'x'}})
        assert d['a.b'] == 'x'

    def test_key_error(self):
        d = self.make_one()
        with pytest.raises(KeyError):
            d['missing']
        with pytest.raises(KeyError):
            d['missing.b']

class TestGetitemProxy(object):
    def make_one(self, wrapped):
        from pyramid_log import _GetitemProxy
        return _GetitemProxy(wrapped)

    def test_proxy(self):
        proxy = self.make_one(MockObject(x=1))
        assert proxy.x == 1
        assert isinstance(proxy, MockObject)

    def test_getitem(self):
        proxy = self.make_one(MockObject(x=42))
        assert proxy['x'] == 42
        assert proxy['missing'] is None

    def test_chained_attribute_access(self):
        proxy = self.make_one(MockObject(x=MockObject(y=42)))
        assert proxy['x.y'] == 42
        assert proxy['missing.y'] is None
        assert proxy['x.missing'] is None

    def test_proxy_none(self):
        proxy = self.make_one(None)
        assert proxy['foo'] is None
        assert isinstance(proxy, type(None))



# -*- coding: utf-8 -*-
#
# Copyright © 2014 Geoffrey T. Dairiki <dairiki@dairiki.org>
#
""" A logging formatter which make pyramid request attributes available
for use in log messages.

"""
from __future__ import absolute_import

import logging
from pyramid.threadlocal import get_current_request
from zope.proxy import ProxyBase

class Formatter(logging.Formatter):
    ''' A logging formatter which makes attributes of the pyramid
    request available for use in its format string.

    Example usage::

        import logging
        import sys
        import pyramid_log

        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(pyramid_log.Formatter(
            "%(asctime)s %(request.unauthenticated_userid)s "
            "[%(request.client_addr)s]\\n"
            "  %(request.method)s %(request.path_qs)\\n"
            "  %(levelname)s %(message)s"))
        root = logging.getLogger()
        root.addHandler(handler)
        root.warning("Say: %s", "howdy")

    which will produce a log message like::

        2014-10-01 17:55:02,001 user.principal [127.0.0.1]
          GET /page?arg=foo
          WARNING Say: howdy

    Passing an explicit request instance
    ====================================

    When generating a log message, the request instance may be passed
    in the ``extra`` dict, e.g.::

        log.warning("Foo: %s", "bar", extra={"request": request})

    If no request is explicitly set,
    :func:`pyramid.threadlocal.get_current_request` is called to
    determine the current request.

    '''
    def format(self, record):
        """ Format the specific record as text.

        This version special in that it makes attributes of the
        pyramid request available for use in the log message.  For
        example, the request method may be interpolated into the log
        message by including ``'%(request.method)s'`` within the format
        string.

        See :meth:`logging.Formatter.format` for further details.

        """
        if hasattr(record, 'request'):
            request = record.request
        else:
            request = get_current_request()

        d = _ChainingDict(record.__dict__)
        d['request'] = _GetitemProxy(request)

        return logging.Formatter.format(self, _ReplaceDict(record, d))

class _ReplaceDict(object):
    """ A minimal object proxy which “replaces” the objects ``__dict__``.

    """
    __slots__ = ['__dict__', '_wrapped']

    def __init__(self, wrapped, d):
        self._wrapped = wrapped
        self.__dict__ = d

    def __getattr__(self, attr):
        return getattr(self._wrapped, attr)

class _ChainingDict(dict):
    """ A dict which supports dotted-key chained lookup.

    Example::

        >>> d = ChainingDict({'a': {'b': 'foo'}})
        >>> d['a.b']                    # same as d['a']['b']
        'foo'


    """
    def __missing__(self, key):
        left, dot, right = key.partition('.')
        if not dot:
            raise KeyError(key)
        try:
            return self[left][right]
        except KeyError:
            raise KeyError(key)

class _GetitemProxy(ProxyBase):
    """ A proxy which adds dict-like read-only access to an object’s
    attributes.

    Attribute chaining is supported using a dotted-key notation.

    Dictionary access to non-existant attributes returns None.

    Example usage::

        >>> d = GetitemProxy(request)

        >>> d['path']                   # gets request.path
        u '/foo'

        >>> d['matched_route.name']     # gets request.matched_route.name
        'foo_route'

        >>> d['unknown_attr']           # Missing attributes => None
        None

    """
    def __getitem__(self, key):
        val = self
        for attr in key.split('.'):
            val = getattr(val, attr, None)
        return val



# -*- coding: utf-8 -*-
#
# Copyright © 2014 Geoffrey T. Dairiki <dairiki@dairiki.org>
#
import sys, os
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

version = "0.1"

install_requires = [
    'pyramid',
    'zope.proxy',
    ]

tests_require = [
    'pytest',
    ]


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ['pyramid_log/tests.py']

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

setup(
    name='pyramid_log',
    version=version,
    description="Include pyramid request attributes in your log messages",
    long_description=README + '\n\n' + CHANGES,
     classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Pyramid",
        "Topic :: System :: Logging",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        ],
    keywords="pyramid logging",
    author="Jeff Dairiki",
    author_email="dairiki@dairiki.org",
    url="http://pypi.python.org/pypi/pyramid_log/",
    license="BSD",
    packages=find_packages(),
    #include_package_data=True,
    zip_safe=True,
    install_requires=install_requires,
    tests_require=tests_require,
    cmdclass = {'test': PyTest},
    )
